# -*- coding: utf-8 -*-

from django.contrib import admin
from django.core.urlresolvers import reverse
from django.utils.safestring import mark_safe
import django.utils.six as six

from .petits_cours_models import PetitCoursDemande, \
    PetitCoursSubject, PetitCoursAbility, PetitCoursAttribution, \
    PetitCoursAttributionCounter
from .models import (
    SurveyQuestionAnswer, SurveyQuestion, CofProfile, EventOption,
    EventOptionChoice, Event, EventCommentField, EventRegistration,
    Survey
)


def add_link_field(target_model='', field='', link_text=six.text_type,
                   desc_text=six.text_type):
    def add_link(cls):
        reverse_name = target_model or cls.model.__name__.lower()

        def link(self, instance):
            app_name = instance._meta.app_label
            reverse_path = "admin:%s_%s_change" % (app_name, reverse_name)
            link_obj = getattr(instance, field, None) or instance
            if not link_obj.id:
                return ""
            url = reverse(reverse_path, args=(link_obj.id,))
            return mark_safe("<a href='%s'>%s</a>"
                             % (url, link_text(link_obj)))
        link.allow_tags = True
        link.short_description = desc_text(reverse_name + ' link')
        cls.link = link
        cls.readonly_fields =\
            list(getattr(cls, 'readonly_fields', [])) + ['link']
        return cls
    return add_link


class SurveyQuestionAnswerInline(admin.TabularInline):
    model = SurveyQuestionAnswer


@add_link_field(desc_text=lambda x: "Réponses",
                link_text=lambda x: "Éditer les réponses")
class SurveyQuestionInline(admin.TabularInline):
    model = SurveyQuestion


class SurveyQuestionAdmin(admin.ModelAdmin):
    search_fields = ('survey__title', 'answer')
    inlines = [
        SurveyQuestionAnswerInline,
    ]


class SurveyAdmin(admin.ModelAdmin):
    search_fields = ('title', 'details')
    inlines = [
        SurveyQuestionInline,
    ]


class EventOptionChoiceInline(admin.TabularInline):
    model = EventOptionChoice


@add_link_field(desc_text=lambda x: "Choix",
                link_text=lambda x: "Éditer les choix")
class EventOptionInline(admin.TabularInline):
    model = EventOption


class EventCommentFieldInline(admin.TabularInline):
    model = EventCommentField


class EventOptionAdmin(admin.ModelAdmin):
    search_fields = ('event__title', 'name')
    inlines = [
        EventOptionChoiceInline,
    ]


class EventAdmin(admin.ModelAdmin):
    search_fields = ('title', 'location', 'description')
    inlines = [
        EventOptionInline,
        EventCommentFieldInline,
    ]


class EventRegistrationAdmin(admin.ModelAdmin):
    list_display = ('__unicode__' if six.PY2 else '__str__', 'event', 'user',
                    'paid')
    list_filter = ('paid',)
    search_fields = ('user__username', 'user__first_name', 'user__last_name',
                     'user__email', 'event__title')


class PetitCoursAbilityAdmin(admin.ModelAdmin):
    list_display = ('user', 'matiere', 'niveau', 'agrege')
    search_fields = ('user__username', 'user__first_name', 'user__last_name',
                     'user__email', 'matiere__name', 'niveau')
    list_filter = ('matiere', 'niveau', 'agrege')


class PetitCoursAttributionAdmin(admin.ModelAdmin):
    list_display = ('user', 'demande', 'matiere', 'rank', )
    search_fields = ('user__username', 'matiere__name')


class PetitCoursAttributionCounterAdmin(admin.ModelAdmin):
    list_display = ('user', 'matiere', 'count', )
    list_filter = ('matiere',)
    search_fields = ('user__username', 'user__first_name', 'user__last_name',
                     'user__email', 'matiere__name')
    actions = ['reset', ]
    actions_on_bottom = True

    def reset(self, request, queryset):
        queryset.update(count=0)
    reset.short_description = "Remise à zéro du compteur"


class PetitCoursDemandeAdmin(admin.ModelAdmin):
    list_display = ('name', 'email', 'agrege_requis', 'niveau', 'created',
                    'traitee', 'processed')
    list_filter = ('traitee', 'niveau')
    search_fields = ('name', 'email', 'phone', 'lieu', 'remarques')


admin.site.register(Survey, SurveyAdmin)
admin.site.register(SurveyQuestion, SurveyQuestionAdmin)
admin.site.register(Event, EventAdmin)
admin.site.register(EventOption, EventOptionAdmin)
admin.site.register(CofProfile)
admin.site.register(PetitCoursSubject)
admin.site.register(PetitCoursAbility, PetitCoursAbilityAdmin)
admin.site.register(PetitCoursAttribution, PetitCoursAttributionAdmin)
admin.site.register(PetitCoursAttributionCounter,
                    PetitCoursAttributionCounterAdmin)
admin.site.register(PetitCoursDemande, PetitCoursDemandeAdmin)
admin.site.register(EventRegistration, EventRegistrationAdmin)
