from django.contrib.auth.models import User, Group
from django.db import models
from django.dispatch import receiver
from django.db.models.signals import post_save, post_delete
from django.utils.translation import ugettext_lazy as _

from cof.petits_cours_models import choices_length

OCCUPATION_CHOICES = (
    ('exterieur', _("Extérieur")),
    ('1A', _("1A")),
    ('2A', _("2A")),
    ('3A', _("3A")),
    ('4A', _("4A")),
    ('archicube', _("Archicube")),
    ('doctorant', _("Doctorant")),
    ('CST', _("CST")),
)


class Profile(models.Model):
    user = models.OneToOneField(
        User,
        on_delete=models.CASCADE,
        related_name="profile"
    )
    login_clipper = models.CharField("Login clipper", max_length=8, blank=True)
    phone = models.CharField("Téléphone", max_length=20, blank=True)
    occupation = models.CharField(_("Occupation"),
                                  default="1A",
                                  choices=OCCUPATION_CHOICES,
                                  max_length=choices_length(
                                      OCCUPATION_CHOICES))
    departement = models.CharField(_("Département"), max_length=50,
                                   blank=True)
    comments = models.TextField(
        "Commentaires visibles par l'utilisateur", blank=True)

    class Meta:
        verbose_name = "Profil"
        verbose_name_plural = "Profils"

    def __str__(self):
        return self.user.username


@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        Profile.objects.get_or_create(user=instance)


@receiver(post_delete, sender=Profile)
def post_delete_user(sender, instance, *args, **kwargs):
    instance.user.delete()


class Club(models.Model):
    ANNUAL = "ANN"
    SEMESTER = "SEM"
    COURSE = "COU"

    COTISATION_FREQUENCY_CHOICES = [
        (ANNUAL, _("Annuel")),
        (SEMESTER, _("Semestriel")),
        (COURSE, _("Au cours"))
    ]

    associations = models.ManyToManyField(Group, related_name="clubs")
    name = models.CharField(_("Nom"), max_length=200, unique=True)
    description = models.TextField("Description", blank=True)
    members = models.ManyToManyField(
        User,
        through="ClubUser",
        related_name="in_clubs",
        blank=True
    )
    price = models.DecimalField(
        _("Cotisation (€)"),
        decimal_places=2,
        max_digits=5,
        blank=True,
        default=0
    )
    cotisation_frequency = models.CharField(
        _("Fréquence de la cotisation"),
        default=ANNUAL,
        choices=COTISATION_FREQUENCY_CHOICES,
        max_length=3,
        blank=True
    )

    def __str__(self):
        template = (
            self.price and "{name} ({price}€ / {cotisation_frequency})"
            or "{name}"
        )
        return template.format(**vars(self))


class ClubUser(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    club = models.ForeignKey(Club, on_delete=models.CASCADE)
    is_respo = models.BooleanField(_("Est responsable du club"))
    has_paid = models.BooleanField(_("A payé sa cotisation"))
